"""Field name constants."""

FIELD_AIP_ID = "AIPID"
FIELD_AIP_NAME = "AIPName"
FIELD_AIP_SIZE = "AIPSize"
FIELD_AIP_UUID = "AIPUUID"
FIELD_AIPS = "AIPs"
FIELD_ALL_AIPS = "AllAIPs"

FIELD_COUNT = "Count"
FIELD_CREATED_DATE = "CreatedDate"
FIELD_CURRENT_LOCATION = "CurrentLocation"

FIELD_DATE_START = "IngestStartDate"
FIELD_DATE_END = "IngestFinishDate"
FIELD_DERIVATIVE_COUNT = "DerivativeCount"
FIELD_DERIVATIVE_FILE = "DerivativeFile"
FIELD_DERIVATIVE_FORMAT = "DerivativeFormat"
FIELD_DERIVATIVE_UUID = "DerivativeUUID"
FIELD_DURATION = "Duration"

FIELD_EVENT = "Event"

FIELD_FILES = "Files"
FIELD_FILE_COUNT = "FileCount"
FIELD_FILE_TYPE = "FileType"
FIELD_FILENAME = "Filename"
FIELD_FORMAT = "Format"
FIELD_FORMATS = "Formats"
FIELD_FORMAT_VERSIONS = "FormatVersions"

FIELD_ID = "ID"
FIELD_INGESTS = "Ingests"
FIELD_INGEST_START_DATE = "IngestStartDate"
FIELD_INGEST_FINISH_DATE = "IngestFinishDate"

FIELD_LOCATIONS = "Locations"
FIELD_LOCATIONS_USAGE_OVER_TIME = "LocationsUsageOverTime"

FIELD_NAME = "Name"

FIELD_ORIGINAL_FILE = "OriginalFile"
FIELD_ORIGINAL_FORMAT = "OriginalFormat"
FIELD_ORIGINAL_NAME = "OriginalName"
FIELD_ORIGINAL_PUID = "OriginalPUID"
FIELD_ORIGINAL_UUID = "OriginalUUID"
FIELD_ORIGINAL_VERSION = "OriginalVersion"

FIELD_PUID = "PUID"

FIELD_RELATED_PAIRING = "RelatedPairing"

FIELD_SIZE = "Size"
FIELD_SIZE_BYTES = "SizeBytes"
FIELD_STORAGE_LOCATION = "StorageLocation"
FIELD_STORAGE_NAME = "StorageName"

FIELD_TRANSFER_NAME = "TransferName"

FIELD_USER = "User"
FIELD_UUID = "UUID"

FIELD_VERSION = "Version"
