{% extends "base.html" %}

{% block content %}

<div class="alert alert-header"><strong>Storage Services</strong></div>

<table id="infotable" class="table table-striped table-bordered">
  <thead>
  <tr>
  <th><strong>Name</strong></th>
  <th><strong>URL</strong></th>
  </tr>
  </thead>
  {% if storage_services %}
    {% for storage_service in storage_services %}
      <tr>
        <td><a href="storage_service/{{ storage_service.id }}">{{ storage_service.name }}</a></td>
        <td>{{ storage_service.url }}</td>
      </tr>
    {% endfor %}
  {% endif %}
</table>
<a href="new_storage_service"><button type="button" class="btn btn-success" style="margin-top: 20px;">New storage service</button></a>
{% endblock %}
