<!-- Ingest log Gantt report. -->

{% extends "report_base.html" %}

{% block content %}

<div class="alert alert-secondary">
  <a class="noprint" onClick="window.print();return false">
    <button type="button" class="btn btn-info" style="float:right;">Print</button>
  </a>
  <strong>User ingest log</strong>
  </br>
  <strong>Storage Service:</strong> {{ storage_service_name }}
  </br>
  {% if storage_location_description %}
    <strong>Location:</strong> {{ storage_location_description }}
    <br>
  {% endif %}
  <strong>Start date:</strong> {{ start_date.strftime('%Y-%m-%d') }}
  <br>
  <strong>End date:</strong> {{ end_date.strftime('%Y-%m-%d') }}
  <br>
  <strong>Number of transfers:</strong> {{ number_of_transfers }}
</div>

  <!-- plotly.express here -->
  {{ plot|safe }}

  <div class="alert alert-warning" role="alert">
    <i class="fa-solid fa-triangle-exclamation fa-lg mr-2" id="2"></i> If no data is visible on the chart above, try specifying a shorter date range on the report page.
  </div>

</div>

{% endblock %}
