import logging
import os
import time
from logging.config import fileConfig

from alembic import context
from flask import current_app
from sqlalchemy.exc import OperationalError

# This is the Alembic Config object.
# It provides access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up the base loggers.
fileConfig(config.config_file_name)
logger = logging.getLogger("alembic.env")


def get_engine():
    try:
        # This works with Flask-SQLAlchemy<3 and Alchemical.
        return current_app.extensions["migrate"].db.get_engine()
    except (TypeError, AttributeError):
        # This works with Flask-SQLAlchemy>=3.
        return current_app.extensions["migrate"].db.engine


def get_engine_url():
    try:
        return get_engine().url.render_as_string(hide_password=False).replace("%", "%%")
    except AttributeError:
        return str(get_engine().url).replace("%", "%%")


# Add your model's MetaData object here for 'autogenerate' support.
# For example, import your model module.
# Set target_metadata = mymodel.Base.metadata.
config.set_main_option("sqlalchemy.url", get_engine_url())
target_db = current_app.extensions["migrate"].db

# Other configuration values required by env.py can be acquired via
# config.get_main_option("my_important_option").
# ... etc.


def get_metadata():
    if hasattr(target_db, "metadatas"):
        return target_db.metadatas[None]
    return target_db.metadata


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(url=url, target_metadata=get_metadata(), literal_binds=True)

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """

    # This callback prevents Alembic from generating empty migrations when
    # there are no schema changes.
    # Reference: http://alembic.zzzcomputing.com/en/latest/cookbook.html
    def process_revision_directives(context, revision, directives):
        if getattr(config.cmd_opts, "autogenerate", False):
            script = directives[0]
            if script.upgrade_ops.is_empty():
                directives[:] = []
                logger.info("No changes in schema detected.")

    conf_args = current_app.extensions["migrate"].configure_args
    if conf_args.get("process_revision_directives") is None:
        conf_args["process_revision_directives"] = process_revision_directives

    connectable = get_engine()
    max_attempts = int(os.getenv("ALEMBIC_CONNECT_RETRIES", "10"))
    retry_delay = float(os.getenv("ALEMBIC_CONNECT_RETRY_DELAY", "3"))

    attempt = 1
    while True:
        try:
            with connectable.connect() as connection:
                context.configure(
                    connection=connection, target_metadata=get_metadata(), **conf_args
                )

                with context.begin_transaction():
                    context.run_migrations()
            break
        except OperationalError as exc:
            if max_attempts and attempt >= max_attempts:
                logger.error(
                    "Could not connect to the database after %s attempts.",
                    attempt,
                )
                raise exc
            logger.warning(
                "Database not ready (attempt %s/%s): %s. Retrying in %.1f seconds.",
                attempt,
                "unlimited" if max_attempts == 0 else max_attempts,
                exc,
                retry_delay,
            )
            time.sleep(retry_delay)
            attempt += 1


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
