{% extends "base.html" %}

{% block content %}

<div class="row">
    <div class="col-md-12 d-flex flex-column justify-content-center align-items-center vh-100">
        <h1>404</h1>
        <h4>Page not found</h4>
	<p>The page you are looking for does not exist.</p>
	<a href="{{ url_for('aggregator.ss_default') }}">Back To Home</a>
    </div>
</div>

{% endblock %}
