.bg-dark {
  background-color: #000 !important;
}

.nav-item {
  margin-left: 20px;
}

body {
  font-family: "Roboto", sans-serif;
}

#console {
  padding: 10px;
  border-radius: 0.25rem;
  color: #0f0;
  font-family: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono",
    "Courier New", monospace;
  font-size: small;
  max-height: 440px;
  overflow-y: auto;
}

.alert-header {
  color: #fff;
  background-color: #17a2b8;
}

@media print {
  .noprint {
    display: none;
  }
}

.report-opts {
  padding-left: 1em;
  display: grid;
  grid-template-columns: repeat(2, 1fr);
  align-items: baseline;
}

.report-opts i {
  font-size: 1.5em;
}

.fixed-width-menu-span-left {
  float: left;
  width: 10em;
}

.fixed-width-menu-span-right {
  float: left;
  width: 7em;
}

.table-header {
  background-color: #ccc !important;
}

/* Use Bootstrap 4 look of button */
.btn-info {
  color: #fff;
  background-color: #17a2b8;
  border-color: #17a2b8;
}
