<!DOCTYPE html>
<html lang="en">
<head>
  <title>{{ storage_service_name }}: file formats count</title>
  <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
  <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='dist/plot_formats_count/plot_formats_count.css') }}">
  <script language="javascript" src="{{ url_for('static', filename='dist/plot_formats_count/plot_formats_count.js') }}"></script>
  <script language="javascript" src="{{ url_for('static', filename='dist/plot_formats_count/plotly.js') }}"></script>
  <script defer type="text/javascript" language="javascript" src="{{ url_for('static', filename='js/reporter/plot_formats_count.js') }}"></script>
</head>
<body>

<div class="container-fluid" style="margin: 20px 0;">

<div class="alert alert-secondary">
  <a class="noprint" onClick="window.print();return false"><button type="button" class="btn btn-info" style="float:right;">Print</button></a>
  <strong>Report: file formats count</strong>
  <br>
  <strong>Storage Service:</strong> {{ storage_service_name }}
  <br>
  {% if storage_location_description %}
    <strong>Location:</strong> {{ storage_location_description }}
    <br>
  {% endif %}
  {{ originalsCount }} original files ingested between {{startdate}} and {{enddate}}
  <br>
  {{ differentFormats }} different formats totalling {{ totalHumanSize }}
</div>

<textarea class="d-none" id="plot_x_axis">{{ x_axis | tojson }}</textarea>
<textarea class="d-none" id="plot_y_axis">{{ y_axis | tojson }}</textarea>
<textarea class="d-none" id="plot_format">{{ format | tojson }}</textarea>
<textarea class="d-none" id="plot_humansize">{{ humansize | tojson }}</textarea>

<div id="scatterplot"></div>

</body>
</html>
