<!DOCTYPE html>
<html lang="en">
<head>
    <title>AIPscan</title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
    <link href="https://fonts.googleapis.com/css2?family=Roboto&display=swap" rel="stylesheet">
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='dist/base/base.css') }}">

    <script type="module" language="javascript" src="{{ url_for('static', filename='dist/base/base.js') }}"></script>

    {% block head_content %}
        <!-- Repeat this block in your template to include additional stylesheets.
            the content in this one will be replaced. -->
    {% endblock %}
</head>
<body data-url-root="{{ request.url_root }}">
{% include "partials/navbar.html" %}

<div class="container-fluid" style="margin: 20px 0;">

  {% with messages = get_flashed_messages() %}
    {% if messages %}
      {% for message in messages %}
        <div class="alert alert-success alert-dismissible" role="alert">
        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">×</span></button>
          {{ message }}
        </div>
      {% endfor %}
    {% endif %}
  {% endwith %}

{% block content %}
{% endblock %}

</div>
</body>
