"""Field name constants."""

FIELD_FILE_FORMAT = "file_format"
FIELD_FILE_TYPE = "file_type"
FIELD_LIMIT = "limit"
FIELD_ORIGINAL_FILES = "original_files"
FIELD_PUID = "puid"
FIELD_STORAGE_LOCATION = "storage_location"
FIELD_START_DATE = "start_date"
FIELD_END_DATE = "end_date"
FIELD_CUMULATIVE = "cumulative"
