<div class="noprint">
  <button type="button" onClick="window.print();return false" class="btn btn-info" style="float:right;">Print</button>
  <button type="button" id="downloadCSV" class="btn btn-info" style="float:right; margin-right: 10px;">Download CSV</button>
</div>
