<!DOCTYPE html>
<html lang="en">
<head>
    <title>AIPscan - Report</title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='dist/base/base.css') }}">

    {% block head_content %}
        <!-- Repeat this block in your template to include additional stylesheets.
            the content in this one will be replaced. -->
    {% endblock %}

    <script type="module" language="javascript" src="{{ url_for('static', filename='dist/base/base.js') }}"></script>
    <script type="module" language="javascript" src="{{ url_for('static', filename='dist/report_base/report_base.js') }}"></script>
</head>

<body data-url-root="{{ request.url_root }}">
  <div class="container-fluid" style="margin: 20px 0;">

    {% block content %}
    {% endblock %}

  </div>
</body>
